#include <pluginlib/class_list_macros.h>
#include <ros/ros.h>

#include "environment/Environment.h"
#include "exceptions/Exceptions.h"

#include "car/camDataMsg.h"
#include "car/environmentDataMsg.h"
#include "car/ussDataMsg.h"

#include <fstream>
#include <vector>
#include <iterator>

PLUGINLIB_EXPORT_CLASS(car::Environment, nodelet::Nodelet);

namespace car
{
Environment::Environment(ros::NodeHandle &nh, std::string &name) :
    nh_(nh)
    , name_(name)
    , distance(0)
    , relativeSpeed(0)
{
    readConfigFile();
}
  
Environment::Environment() :
    distance(0)
    , relativeSpeed(0)
{
    readConfigFile();
}
  
Environment::~Environment() {}
 
void Environment::readConfigFile()
{
    // open config file
    std::string userHome = getenv("HOME");
    std::ifstream configFile;
    configFile.open(userHome + "/.CarConfig/environment.config", std::ifstream::in);
    if (!configFile.is_open()) { throw FileNotFound(); }

    // desired parameters
    
    // read file
    std::string contentLine;
    while (!configFile.eof())
    {
        std::getline(configFile, contentLine);
        
        // split this line
        std::istringstream iss(contentLine);
        std::vector<std::string> words {std::istream_iterator<std::string>(iss), {} };

        if ( words.size() > 1 ) // this line contains parameter[at (0)] and value [at (1)]
        {
            if (words.at(0) == "x1:") {
                x(0) = std::stod(words.at(1));
            } else if (words.at(0) == "x2:"){
                x(1) = std::stof(words.at(1));
            } else if (words.at(0) == "p11:"){
                P(0, 0) = std::stof(words.at(1));
            } else if (words.at(0) == "p12:"){
                P(0,1) = std::stof(words.at(1));
            } else if (words.at(0) == "p21:"){
                P(1,0) = std::stof(words.at(1));
            } else if (words.at(0) == "p22:"){
                P(1,1) = std::stof(words.at(1));
            } else if (words.at(0) == "f11:"){
                F(0,0) = std::stof(words.at(1));
            } else if (words.at(0) == "f12:"){
                F(0,1) = std::stof(words.at(1));
            } else if (words.at(0) == "f21:"){
                F(1,0) = std::stof(words.at(1));
            } else if (words.at(0) == "f22:"){
                F(1,1) = std::stof(words.at(1));
            } else if (words.at(0) == "q11:"){
                Q(0,0) = std::stof(words.at(1));
            } else if (words.at(0) == "q12:"){
                Q(0,1) = std::stof(words.at(1));
            } else if (words.at(0) == "q21:"){
                Q(1,0) = std::stof(words.at(1));
            } else if (words.at(0) == "q22:"){
                Q(1,1) = std::stof(words.at(1));
            } else if (words.at(0) == "h1:"){
                H(0) = std::stof(words.at(1));
            } else if (words.at(0) == "h2:"){
                H(1) = std::stof(words.at(1));
            } else if (words.at(0) == "r:"){
                R(0) = std::stof(words.at(1));
            }       
        }
    }
    configFile.close();
    return;
}

void Environment::onInit()
{
    NODELET_INFO("Environment::onInit -- START");
    environmentData = nh_.advertise<environmentDataMsg>("environmentData", 1);
    ussData = nh_.subscribe("ussData", 1, &Environment::ussDataCallback, this);
    camData = nh_.subscribe("camData", 1, &Environment::camDataCallback, this);
    NODELET_INFO("Environment::onInit -- END");
}

void Environment::ussDataCallback(const ussDataMsg::ConstPtr& inMsg)
{
    // std::cout << "Environment recived new uss data (" << inMsg->distance << ").\n";
    distance = inMsg->distance;
    relativeSpeed = 1.0;
    
    environmentDataMsg outMsg;
    outMsg.distance = distance;
    outMsg.relativeSpeed = relativeSpeed;
    
    environmentData.publish(outMsg);
}

void Environment::camDataCallback(const camDataMsg::ConstPtr& inMsg)
{
    // std::cout << "Environment recived new cam data ( ).\n";
}
}
